;DirectKeyer    G4JNT Jan 2021
;  Replacement for Keyer_12F derived directly from KEYER 4


;	Programming mode at boot up, same protocol as KEYER12F
;       Auto selection of Serial Interface polarity

; Enter prog mode if MODESW is pressed at turn on
; LED Flashes Fast until 1s after it is released
;
	list		P=12F617
	include		"P12F617.INC"

	errorlevel 	2
	__config  0x3FC4  ;11 1111 0100 0010    WrtProt Off   BO EN  Not CP  MCLR Internal   PWRT EN  WDT Dis  HS Osc
						;0x3F42 for XT, 3FC4 for INTRC 8MHz
	;PWM Output on CCP1  	GPIO, 2

;--------------------- Define these user values -------------------------------------


	TIMINT		=	8		;Timer interval, ms
	PULSEONTIM	=	d'80'		;Pulse on period, ms   max value 128

;Assembler values calculated using 32 bit {signed} integer arithmetic from user values, -- Do not touch
	FOSC		=	d'2000000'	
	FINT		=	FOSC / 4 / d'64'
	INTSPERTIME	=	(TIMINT * FINT) / d'1000'
	PULSEON		=	PULSEONTIM / TIMINT

	#define		SERIN	GPIO, 0		;Shared with
	#define		STROBE	GPIO, 0		;Keep in for legacy
	#define		MODESW	GPIO, 1		;Shared with:
	#define		SEROUT	GPIO, 1
	#define		KEYOUT	GPIO, 2
	#define		LED	GPIO, 4		;
	#define		PTT	GPIO, 5

	#define		SaveTermFlag	Flags, 4	;Used in writing to flash
	#define		SerPol		Flags, 5	;1 = RS232 polarity, 0 = TTL

;=====================================
 org 0
boot    nop
        goto    Startup        		

;=========================
 org 4			;TMR2 Postscale interrupt rate.
	bsf	STROBE		;Changes 
	bcf	PIR1, TMR2IF

	movwf   Wreg            ;Save W
	swapf   STATUS , W      ;
	movwf   StsReg          ;Stave STATUS, without affecting it
				;Increment the timing counter
	incf	IntCount	
	movlw	INTSPERTIME
	subwf	IntCount, W	
	btfsc	STATUS, C	
	clrf	IntCount	

	btfsc	STATUS, Z	
	incf	TimCountLo			;Counts at Basic timing interval 5ms

	swapf   StsReg , W      ;Restore STATUS
	movwf   STATUS
	swapf   Wreg            ;Restore W
	swapf   Wreg , W        ;
	bcf	STROBE
	retfie		
;---------------------------------------------------------
Startup
    	BANKSEL	ANSEL
	clrf	ANSEL			;WHY WHY WHY WHY WHY !!!!!!!!!!!!!!!!!!!!!
 
    	movlw   b'00001010'     	;0 STROBE, 1 MODE SW.   These will swap I/O in Prog mode
    	movwf   TRISIO           	;2 PWM, 3 KEY, 4 LED, 5 PTT
	movwf	WPU  
    	movlw   b'00000000'      	;Pull-ups
    	movwf   OPTION_REG

	movlw	0x3F			;PWM Period = Fclk/64/4 = 7812.5Hz
	movwf	PR2			;

	bsf	PIE1 , TMR2IE		;TMR2 overflow
    	BANKSEL	GPIO

	movlw	b'00011100'		;TMR2 On, prescale 1, postscale 4
	movwf	T2CON

	clrf	Flags
	clrf	GPIO

	clrf	DelCount
	clrf	DelCount2

	movlw	PULSEON	
	movwf	CWSpeed

	btfss	MODESW			;MODESW pressed at turn on 
	goto	ProgMode

	bsf	INTCON, PEIE		;Peripheral Interrupt enable
	bsf	INTCON, GIE		;Start it running
;.......................................
MainLoop		
	clrf    CWMsgCnt
	bsf	PTT
	movlw 	HIGH(CWContents)
	movwf	Temp2
	movlw	LOW(CWContents)
	call    ReadFlash	;Test for terminator at start = no CW message

	addlw   0
	btfsc   STATUS, Z
	goto	MainLoop

	movlw   " "
	call    SendCW          ;Initial space
CWLoop
	movlw 	HIGH(CWContents)
	movwf	Temp2

	movf    CWMsgCnt, W
	addlw	LOW(CWContents)
	call    ReadFlash
	addlw   0
	btfsc   STATUS, Z	;Test for terminating zero
	goto	MainLoop	;And if so, restart

	call    SendCW
	incf    CWMsgCnt
	goto    CWLoop
;==============================================
SendCW          ;Sends a single CW character
    movwf   CWChar
    btfsc   CWChar, 7       ;Test for control code
    goto    DecodeToken

    movlw   HIGH(CWTable)
    movwf   PCLATH

    movf    CWChar, W
    addlw   0xE0     ;Subtract 32 (add 256 - 32) to point to table
    call    CWTable

    movwf   CWChar      ;Doesn't affect STATUS so...
    movf    CWChar      ;  check for invalid returned code here
    btfsc   STATUS , Z      
    goto    NoChar

    movlw   0x07    
    andwf   CWChar , W      ;Extract element count into W
    movwf   Counter     
    movlw   6           
    subwf   Counter , W     ;W = Elcount - 6.  If +ve/0 then C=1, > 5 els
    btfss   STATUS , C
    goto    Send5
    
    rrf     Counter     ;Puts LSB into C as 110 = .  111 = -
    call    SendEle     ;First  . or -  for 6 element characters
    movlw   5
    movwf   Counter     ;Force count to 5 for rest of the elements
Send5           
    rlf     CWChar      ;Element to be sent into C.
    call    SendEle     ;   count already in Counter
    decfsz  Counter    
    goto    Send5
NoChar                  ;Inter word gap, and invalid codes.
    bcf     KEYOUT    ;Just in case
    bcf     LED
    call    CWDelay     
    call    CWDelay
    return
;.........
DecodeToken
    movlw   HIGH(CWTable)
    movwf   PCLATH
    btfss   CWChar, 6   ;B6 0 = Delay mode, 1 = WPM
    goto    ProgDelay

    movf    CWChar, W   ;11xxxsss  sss = WPM code
    andlw   0x07        ;Mask out speed control bits
    call    CWCtrlTable
    movwf   CWSpeed
    return
;.............
ProgDelay

    btfss   CWChar, 5   ;Set Tx/Rx line
    bcf     PTT
    btfsc   CWChar, 5
    bsf     PTT

    btfss   CWChar, 4   ;Set keying line
    bcf     KEYOUT
    btfsc   CWChar, 4
    bsf     KEYOUT
 
    bsf     LED         ;LED off if delay with Key up or in Rx mode
    btfss   KEYOUT
    bcf     LED
    btfss   CWChar, 5   ;
    bcf     LED

    movf    CWChar, W
    andlw   0x07
    addlw   0x08        	;Point to delay settings in table
    call    CWCtrlTable		;Return with number of seconds of delay in W

	movwf	DelCount
OuterDelLoop

	clrf	TimCountLo
InnerDelLoop			;Basic one second delay
	movlw	d'125'
	subwf	TimCountLo, W
	btfss	STATUS, C
	goto	InnerDelLoop	

	decfsz	DelCount
	goto	OuterDelLoop
QuickOut
	bcf	KEYOUT
	bcf	LED
	bsf	PTT		
 	return
;-----------------------------------
SendEle
	bsf	KEYOUT
	bsf	LED
	btfss	STATUS, C
	goto	SendDot
	call	CWDelay
	call	CWDelay
SendDot
	call	CWDelay
	bcf	KEYOUT
	bcf	LED
	call	CWDelay

	return
;------------------------------------------
CWDelay
	clrf	TimCountLo
DeLoop
	movf	CWSpeed, W
	subwf	TimCountLo, W
	btfss	STATUS, C		
	goto	DeLoop	

	return
;--------------------------------------
ReadFlash				;Read defined memory location as byte
	bcf	INTCON, GIE
	BANKSEL	PMADRL	
	movwf	PMADRL		;Low address byte in W,  high byte defined from label

 	BANKSEL	Temp2
 	movf	Temp2, W

 	BANKSEL	PMADRH
 	movwf	PMADRH
 
	bsf	PMCON1, RD	;Do the read
	nop			;This can be replaced by an instruction;  any ideas?
	nop			;This one CAN'T be.
	movf	PMDATL, W	;Low Byte contents returned in W.
	BANKSEL	GPIO		;As good as anything to get back to Page 0
	BSF	INTCON, GIE
	return
;---------------------------------------
SaveFlash			;Saves contents of the buffer up to zero terminator
	BANKSEL	PMADRL		;Only entered in ProgMode so interrupt won't be running
	movlw	HIGH(CWContents)
	movwf	PMADRH
	movlw	LOW(CWContents)
	movwf	PMADRL		;
	
	movlw	BUFFER
	movwf	FSR
SaveLoop
	movf	INDF, W		;
	movwf	PMDATL
	incf	FSR

	movf	PMDATL		;Test for terminator
	btfsc	STATUS, Z	;Freeze buffer at zero terminator point
	decf	FSR		; so INDF can be tested for zero to end

	clrf	PMDATH		;Only saving 8 bit data
	bsf	PMCON1, WREN

	movlw	0x55
	movwf	PMCON2
	movlw	0xAA
	movwf	PMCON2
	bsf	PMCON1, WR
	nop
	nop
	bcf	PMCON1, WREN

	movf	PMADRL, W
	incf	PMADRL
	andlw	0x03
	sublw	0x03
	btfss	STATUS, Z
	goto	SaveLoop	;Saves 4 bytes

	movf	PMDATL
	btfss	STATUS, Z
	goto	SaveLoop	;Gets out when zero terminator is found

	BANKSEL	GPIO		

	return
;---------------------------------------
;---------------------------------------
;================~~~~~~~~~~~~~~~~======================~~~~~~~~~~~~~~~~~~~~~~===================~~~~~~~~~
ProgMode		;Here when KEY and MODESW both pressed at turn on
	movlw	d'15'		;Fast flash LED for 1.5s and until both lines go high
	movwf	Counter		; go high to show progmode
ProgSLoop	
	bcf	LED
	movlw	d'50'
	call	DelayNms
	bsf	LED
	movlw	d'50'
	call	DelayNms
	decfsz	Counter
	goto	ProgSLoop

	btfss	MODESW		;Wait for key to go high
	goto	ProgMode
				;GPIO0/1 switched I/O as MODESW needs to become an output
	BANKSEL TRISIO		;  so it isn't loaded by COM port
	movlw   b'00001001'     ;0 SERIN, 1 SEROUT
    	movwf   TRISIO          ;
	BANKSEL	GPIO

	movlw	d'100'		
	call	DelayNms

	bcf	SerPol		;Test resting polarity of SERIN line  Low = RS232   High = TTL
	btfss	SERIN		;O/C pulled high so default is TTL
	bsf	SerPol		;If low it's RS232	

	btfss	SerPol		;Set SEROUT to the correct polarity
	bsf	SEROUT		
	btfsc	SerPol
	bcf	SEROUT		;Then delay a bit to clear any BREAK state in terminal
	movlw	d'250'		
	call	DelayNms

	call	CrLf
	call	CrLf
	movlw   LOW(IntroMsg)
	call	DispMsg
ProgDoLoop
	movlw   LOW(MenuPrompt)
	call    DispMsg
	call	Qprompt

	call    WaitData        ;Get D / E / S commands
	movf	RxChar, W
	call	Send232
	call	CrLf
 
	movlw   "D"
	subwf   RxChar, W
	btfsc   STATUS, Z
	goto    Display

	movlw   "E"
	subwf   RxChar, W
	btfsc   STATUS, Z
	goto    EnterMsg
	
	movlw   "S"
	subwf   RxChar, W
	btfsc   STATUS, Z
	goto    ReStart

	goto    ProgDoLoop      ;Any other character
;.................
ReStart
	call	CrLf
	call	CrLf

	BANKSEL	TRISIO			;Restore the I/O pins
   	movlw   b'00001010'     	;0 STROBE, 1 MODE SW.   These will swap I/O in Prog mode
    	movwf   TRISIO           	;2 PWM, 3 KEY, 4 LED, 5 PTT
	movwf	WPU  
	BANKSEL	GPIO

	bsf	INTCON, PEIE		;External Interrupt enable
	bsf	INTCON, GIE		;Start it running

	goto	MainLoop
;-----------------
Display
    	call    CrLf
    	movlw   " "
    	call    Send232
    	movlw   "C"
    	call    Send232
    	movlw   "W"
    	call    Send232
    	movlw   " "
    	call    Send232
    	movlw   " "
    	call    Send232
    	movlw   0x22
    	call    Send232

    	movlw   0
    	movwf   CWMsgCnt
DispCWLoop
	movlw 	HIGH(CWContents)
	movwf	Temp2
    	movf    CWMsgCnt, W
	addlw	LOW(CWContents)
   	call    ReadFlash
    	addlw   0
    	btfsc   STATUS, Z       ;Test for terminator
    	goto    DoneDisp

    	movwf   Token           ;Save in case it is a Token
    	btfss   Token, 7
    	goto    NotToken
    	call    DispToken      

NotToken
    	call    Send232
    	incf    CWMsgCnt
    	goto    DispCWLoop
;...........
DoneDisp
    	movlw   0x22
    	call    Send232         ;Closing "
    	call    CrLf
    	call    CrLf
    	goto    ProgDoLoop
;-----------------------
EnterMsg                    ;Real time data entry and write to Flash via buffer
	movlw	BUFFER
	movwf	FSR		;Start of Buffer memory
	
	clrf    CWMsgCnt
	movlw   LOW(CWPrompt)
	call    DispMsg

	movlw   LOW(SpdMsg1)
	call    DispMsg
	movlw   LOW(SpdMsg2)
	call    DispMsg

	call	Qprompt

EnterCWMsgLoop
    call    WaitData
    movlw   0x0D
    subwf   RxChar, W
    btfsc   STATUS, Z
    goto    CWMsgStored

    movlw   0x08            ;Abort
    subwf   RxChar, W
    btfsc   STATUS, Z
    goto    Abort

    movlw   " "             ;Test for all other control characters
    subwf   RxChar, W       ;  and ignore them
    btfss   STATUS, C       ;If +ve or 0, C=1 > 0x31 so OK
    goto    EnterCWMsgLoop

    movlw   "<"
    subwf   RxChar, W
    btfsc   STATUS, Z
    goto    GetTokenInfo
    
    movf    RxChar, W
    call    Send232
    movf    RxChar, W
BackHere                    ;return from token entry decoding to this point
	movwf	INDF		;Save in buffer
	incf	FSR

	btfsc	FSR, 7		;Look for overrun to 0x80
	goto	OverRun

	incf	CWMsgCnt
	goto	EnterCWMsgLoop
;...........
OverRun
	decf	FSR
	clrf	INDF		;Add a terminator for safety
	movlw	LOW(OVRunPrompt)
	call	DispMsg
	goto    CWMsgStored
;................
Abort                    ;
	clrf	INDF		;Add terminators for safety
	incf	FSR
	clrf	INDF
	call	CrLf
	goto    Display
;..................
CWMsgStored
	movlw   0               ;Counter has already been incremented
	movwf	INDF		; so save terminator

;	call	CrLf		;Display data from RAM (tokens not decoded)
	movlw	BUFFER
	movwf	FSR
DispMemLoop
	movf	INDF, W
	iorlw	0
	btfsc	STATUS, Z
	goto	DoneShowEntry

	incf	FSR

	goto	DispMemLoop
;.......
DoneShowEntry
	call	CrLf
	call    SaveFlash        
	goto    Display     ;Return to main prog menu
;........................................
;CW Delay bit coding    10RKxddd   R - 0=Rx, K - 0=KeyUp  ddd = delay code
;CW WPM Bit Coding      11xxxsss  sss = WPM code      

GetTokenInfo   ; From RS232 eg <DTDA>   Delay, Transmit, key Down, duration A
    movlw   "<"     ;       or <WC>     WPM, speed C
    call    Send232         ;Echo back token intro
    movlw   0x80
    movwf   Token           ;Base value

    call    WaitData        ;First look for WPM or Delay code
    movlw   "W"
    subwf   RxChar, W
    btfsc   STATUS, Z
    goto    SetWPM

    movlw   "D"             ;Echo back what S/W has interpreted it as...
    call    Send232

    call    WaitData        ;Get Tx/Rx status
    movlw   "T"
    subwf   RxChar, W
    btfsc   STATUS, Z
    bsf     Token, 5        ;B5 = 0 Rx, 1 Tx

    movlw   "R"
    btfsc   Token, 5
    movlw   "T"
    call    Send232         ;Echo back what is actually stored

    call    WaitData        ;Get Key Up/Down
    movlw   "D"
    subwf   RxChar, W
    btfsc   STATUS, Z
    bsf     Token, 4        ;B4 = 0 Up, 1 Dn

    movlw   "U"
    btfsc   Token, 4
    movlw   "D"
    call    Send232         ;Echo back what is actually stored

    call    WaitData        ;Get delay code A-H
    addlw   0xBF            ;Subtract 0x41 to convert to 0 - 7
    andlw   0x07            ;Just in case
    addwf   Token

    addlw   "A"             ;Return code for what it actually is
    call    Send232
    goto    WaitCloseToken
;..........
SetWPM
    movlw   "W"
    call    Send232
    bsf     Token, 6

    call    WaitData        ;Get delay code A-H
    addlw   0xBF            ;Subtract 0x41 to convert to 0 - 7
    andlw   0x07            ;Just in case
    addwf   Token

    movf    Token, W
    andlw   0x07
    addlw   "A"
    call    Send232

WaitCloseToken
    call    WaitData        ;Wait for closing >
    movlw   ">"
    subwf   RxChar, W
    btfss   STATUS, Z
    goto    WaitCloseToken
    movlw   ">"
    call    Send232
    movf    Token, W
    goto    BackHere        ;Return with Token code in W
;-------------------------------------

;===========  SUBROUTINES SPECIFIC TO PROG MODE BELOW HERE ============================
DelayNms
	movwf	DelCount2
DelayNLoop
	call	Delay1ms
	decfsz	DelCount2
	goto	DelayNLoop
	return
;----------------------------------------------
Delay1ms
	movlw	(FOSC / d'10000')	;Loop is 10.N clocks long
	movwf	DelCount
Del1msLoop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	decfsz	DelCount
	goto	Del1msLoop

	return
;---------------------------------------
CrLf                    ;Uses less space than storing in string
    movlw   0x0D
    call    Send232
    movlw   0x0A
    call    Send232
    return
;--------------------------------
Qprompt
	movlw	"?"
	call	Send232
	movlw	" "
	call	Send232
	return
;---------------------------

DispToken          ;If valid, decodes token byte to RS232 link
    movlw   "<"
    call    Send232
    btfss   Token, 6
    goto    ShowDelay

    movlw   "W"
    call    Send232
    movf    Token, W
    andlw   0x07
    addlw   "A"
    call    Send232

    goto    OutDispToken
;..........
ShowDelay
;CW Delay bit coding    10RKxddd   R - 0=Rx, K - 0=KeyUp  ddd = delay code
    movlw   "D"
    call    Send232

    movlw   "R"
    btfsc   Token, 5
    movlw   "T"
    call    Send232

    movlw   "U"
    btfsc   Token, 4
    movlw   "D"
    call    Send232

    movf    Token, W
    andlw   0x07
    addlw   "A"
    call    Send232

OutDispToken
    movlw   ">"             ;A Send232 command always follows the return
    return
;---------------------------
WaitData              		;Sit in here waiting for RS232 start bit
	btfss	SerPol
	goto	TestForTTL
	
	btfss	SERIN		;Test for RS232 start transition  0 >> 1
	goto	WaitData
	goto	TransFound	
;.............
TestForTTL
	btfsc	SERIN		;Test for TTL start transition   1 >> 0
	goto	WaitData
TransFound

	bcf     LED             ;Blink LED for each character entered
	call    HalfBitDelay    

	btfss	SerPol
	goto	TestForTTL2
	
	btfss	SERIN		
	goto	WaitData
	goto	TransFound2	
;.............
TestForTTL2
	btfsc	SERIN		
	goto	WaitData
TransFound2
    movlw   8               ;
    movwf   Counter
    clrf    RxChar          ; to here from transition = half bit
ByteLoop
    call    HalfBitDelay
    call    HalfBitDelay    
    bcf     STATUS , C     
    btfss   SERIN         
    bsf     STATUS , C     
    rrf     RxChar        
    decfsz  Counter
    goto    ByteLoop        ;this loop 2 * HalfBitDelay + 8

	btfss	SerPol		;Invert if TTL data
	comf	RxChar

    movf    RxChar , w      ;Returns with stop bit+  slack period
    movlw   "a"             ;Convert to Upper case
    subwf   RxChar, W       ;W = Char - "a"
    btfss   STATUS, C       ;if-ve (C=0) char < "a" so ignore
    goto    UpCaseDone

    movf    RxChar, W
    sublw   "z"             ;W = "z" - Char
    btfss   STATUS, C       ;if -ve (C=0)  char > "z" so ignore
    goto    UpCaseDone

    bcf     RxChar, 5       ;Reset bit 5 to force upper case
UpCaseDone
    movf    RxChar, W
    bsf     LED
    return                  ;received byte in RxChar and W
;-------------------------
HalfBitDelay                ;For RS232 1200 Baud link
    movlw   FOSC / d'12000' - 4         ;
    movwf   DelCount
HalfBaudDelLoop
    nop
    nop
    decfsz  DelCount
    goto    HalfBaudDelLoop
    return
;---------------------------
Send232             ;Enter with data in W, send it to PC
	movwf   Temp
	btfss	SerPol
	comf	Temp		;TTL polarity is inverted

	btfsc	SerPol		;1 = RS232, 0 = TTL
	bsf     SEROUT 		;start bit
	btfss	SerPol
	bcf	SEROUT

    call    HalfBitDelay
    call    HalfBitDelay        ;
    movlw   8
    movwf   Counter
    nop
    nop
SendDatLoop
    rrf     Temp        	;bit into C
    btfsc   STATUS , C
    bcf     SEROUT
    btfss   STATUS , C
    bsf     SEROUT         	;could have 2us jitter depending on 1/0
    call    HalfBitDelay
    call    HalfBitDelay    
    decfsz  Counter
    goto    SendDatLoop     	;loop 8 + HalfBitDelay long

	btfsc	SerPol		;1 = RS232, 0 = TTL
	bcf     SEROUT 		;Stop bit
	btfss	SerPol
	bsf	SEROUT
    call    HalfBitDelay    
    call    HalfBitDelay    
    call    HalfBitDelay    
    call    HalfBitDelay    ;Two stop bits
    return
;-------------------------

;Generating a message from data stored in programme memory
;               Uses AddrHold, MsgCount

DispMsg   ;Enter with address of Text in W  eg.  'MOVLW LOW(TextLabel)'
    movwf   AddrHold    ;Store offset of text string

    movlw   HIGH(MsgTable)
    movwf   PCLATH

    movlw   low(MsgTable)
    addlw   1
    subwf   AddrHold , W    ;W = AddrTextString - TableStart - 1
                            ;  = Offset of string start in table
MsgLoop
    movwf   MsgCount
    call    MsgTable
    andlw   0xFF            ;end of message
    btfsc   STATUS , Z
    goto    MsgDone

    call    Send232
    movf    MsgCount, W
    addlw   1
    goto    MsgLoop

MsgDone
    call    CrLf        ;Always put a CR LF at end, saves storing it in msg.
    return
;.........
;============================================
 ORG 0x300
MsgTable
    addwf   	PCL
IntroMsg    	dt "G4JNT Beacon Keyer" , 0
CWPrompt    	dt "Token Codes" , 0
MenuPrompt  	dt "[D]isplay [E]nter [S]end",0
OVRunPrompt 	dt " MEM OVRN",0
SpdMsg1		dt "Delay seconds A-1 B-5 C-10 D-15 E-20 F-30 G-60 H-90",0
SpdMsg2		dt "WPM           A-6 B-8 C-10 D-12 E-15 F-20 G-24 H-30",0

MsgTableEnd

    if ((DispMsg & 0xFF) >= (MsgTableEnd & 0xFF))
        ERROR    "Table code crosses page boundary"
        ERROR    "ADJUST START OF TABLE"
    endif


;=================================================

 


;----------------------------
 org 0x400
CWTable           ;CW Characters in compressed form. 
    addwf   PCL
    
    dt   b'00000000'  ; [sp] 00
    dt   b'01011111'  ;  !   5F
    dt   b'10010110'  ;  "   96
    dt   b'00010101'  ;  #   15
    dt   b'00000000'  ;  $   00
    dt   b'00000000'  ;  %   00
    dt   b'01000101'  ;  &   45
    dt   b'11110110'  ;  '   F6
    dt   b'10110101'  ;  (   B5
    dt   b'01101111'  ;  )   6F
    dt   b'01000101'  ;  *   45
    dt   b'01010101'  ;  +   55
    dt   b'10011111'  ;  ,   9F     Coding for CW characters of up to
    dt   b'00001111'  ;  -   0F      6 elements.  Stored as :
    dt   b'10101110'  ;  .   AE     Bits 7 to 3 make first five CW elements
    dt   b'10010101'  ;  /   95     Reading  Left to Right 
    dt   b'11111101'  ;  0   FD     Coded as 0 = dit   1 = dah
    dt   b'01111101'  ;  1   7D     Bts 2-0 are number of elements except for 
    dt   b'00111101'  ;  2   3D     110 > 1st element is a dit  
    dt   b'00011101'  ;  3   1D     111 > 1st element is a dah       
    dt   b'00001101'  ;  4   0D
    dt   b'00000101'  ;  5   05
    dt   b'10000101'  ;  6   85
    dt   b'11000101'  ;  7   C5
    dt   b'11100101'  ;  8   E5
    dt   b'11110101'  ;  9   F5
    dt   b'11000111'  ;  :   C7
    dt   b'01010111'  ;  ;   47
    dt   b'00000000'  ;  <   00
    dt   b'10001101'  ;  =   8D
    dt   b'00000000'  ;  >   00
    dt   b'01100110'  ;  ?   66
    dt   b'11010110'  ;  @   D6
    dt   b'01000010'  ;  A   42
    dt   b'10000100'  ;  B   84
    dt   b'10100100'  ;  C   A4
    dt   b'10000011'  ;  D   83
    dt   b'00000001'  ;  E   01
    dt   b'00100100'  ;  F   24
    dt   b'11000011'  ;  G   C3
    dt   b'00000100'  ;  H   04
    dt   b'00000010'  ;  I   02
    dt   b'01110100'  ;  J   74
    dt   b'10100011'  ;  K   A3
    dt   b'01000100'  ;  L   44
    dt   b'11000010'  ;  M   C2
    dt   b'10000010'  ;  N   82
    dt   b'11100011'  ;  O   E3
    dt   b'01100100'  ;  P   64
    dt   b'11010100'  ;  Q   D4
    dt   b'01000011'  ;  R   43
    dt   b'00000011'  ;  S   03
    dt   b'10000001'  ;  T   81
    dt   b'00100011'  ;  U   23
    dt   b'00010100'  ;  V   14
    dt   b'01100011'  ;  W   63
    dt   b'10010100'  ;  X   94
    dt   b'10110100'  ;  Y   B4
    dt   b'11000100'  ;  Z   C4

CWCtrlTable
    	addwf   PCL
    	dt  d'25',d'19',d'15',d'12',d'10',d'8',d'6',d'5'		;Units of 8ms
;	     6       8     10     12    15    20    24   30  WPM
    	dt  d'1',d'5',d'10',d'15',d'20',d'30',d'60',d'90'		;Seconds

 org 0x0460
		;Write process does 16 at a time, so addresses need to be apart
CWContents		;CW Message up to ~50 chars stored here.
	dt	b'11000101',"G4JNT BCN KEYER",b'10110001', 0   	;<WE>    <DTDB>


	dt	"&&&&&&&& KEYER4 &&&&&&&&"		
;--------------------------------------------------
    cblock	0x20		;12F617 has 128 bytes RAM
  	Wreg, StsReg
	Temp, Temp2
	Counter
	IntCount, TimCountLo
	DelCount, DelCount2
 	Flags
	A0, A1, B0, B1
	CWChar, CWSpeed, CWMsgCnt
	RxChar, Token, AutoEEADR
	MsgCount, AddrHold
	BUFFER			;Used as a label only. Must be last item as buffer grows down from here
    endc


	end