;KEYER3    PWM Manual CW, DOTS and Chirp 
;  Key input, Mode select pushbutton.
;Generates chirp or three alternating tones, passes key through using middle tone.

;PWM rate = Fxtal / 256 (8 bit PWM resolution).   Fosc = 8MHz INTRC
;Sampling rate = PWM rate /4 (Postscale = 4)  7.8kHz
;24 Bit DDS

	list		P=12F615
	include		"P12F615.INC"

	errorlevel 	2
	__config  0x3FC4  ;11 1111 0100 0010    WrtProt Off   BO EN  Not CP  MCLR Internal   PWRT EN  WDT Dis  HS Osc
						;0x3F42 for XT, 3FC4 for INTRC 8MHz
	;PWM Output on CCP1  	GPIO, 2

;--------------------- Define these values
	FCLK		=	d'2000000'
	TONE1		=	d'494'
	TONE2		=	d'523'
	TONE3		=	d'587'
	STARTTONE	=	d'350'
	ENDTONE		=	d'2750'
	CHIRPRATE	=	d'4000'		;Hz/s
	TIMINT		=	8		;Timer interval, ms
	TMOX		=	d'800'		;CW MOX delay, ms
	CYCLE	=	d'320'			;rate of tone cycling ms

;-------- Assembler values then calculated using {signed} 32 bit integer arithmetic to best resolution possible
	FINT		=	FCLK / 4 / d'64'		;Prescale = 4
	TONE1FREQ 	=	(TONE1 * d'1048576') / FINT * d'16'
	TONE2FREQ 	=	(TONE2 * d'1048576') / FINT * d'16'
	TONE3FREQ 	=	(TONE3 * d'1048576') / FINT * d'16'
	STARTFREQ 	=	(STARTTONE * d'1048576') / FINT * d'16'
	ENDFREQ 	=	(ENDTONE * d'65536') / FINT * d'256'
	CHRATE		=	((CHIRPRATE * d'65536') / FINT) * d'256' / FINT
	INTSPERTIME	=	(TIMINT * FINT) / d'1000'
	MOX		=	TMOX / TIMINT
	CYCLETIM	=	CYCLE / TIMINT

	#define		SERIN	GPIO, 0		;Shared with
	#define		STROBE	GPIO, 0		;Keep in for legacy
	#define		MODESW	GPIO, 1		;Shared with:
	#define		SEROUT	GPIO, 1
;			PWM	GPIO, 2
	#define		KEY	GPIO, 3		;Shared with PGM, so needs diodes
	#define		LED	GPIO, 4		;
	#define		PTT	GPIO, 5

	#define 	ModeFlag	Flags, 1	;0 = Tone, 1 = Chirp
	#define		ManualKey	Flags, 2
	#define		KeyState	Flags, 3

;=====================================
 org 0
boot    nop
        goto    Startup        		

;=========================
 org 4			;TMR2 Postscale interrupt rate.  Routine has to complete in 256 clocks total
	bsf	STROBE
	bcf	PIR1, TMR2IF

	movwf   Wreg            ;Save W
	swapf   STATUS , W      ;
	movwf   StsReg          ;Stave STATUS, without affecting it
	movf	PCLATH, W
	movwf	PCLcache	;Save PCLATH


	movlw	HIGH(SineTabContents)
	movwf	PCLATH

	movf	FreqHi, W
	call	SineTable

	movwf	A0
	movf	Amplitude, W
	movwf	B0
	call	Multiply8	;87 clocks, constant delay routine, MSB result in Q1

	bcf	STATUS, C
	rrf	Amplitude, W	;Add [128 - Amplitude/2] to normalise
	sublw	0x80		;   the level to mid point
	addwf	Q1, W

	movwf	A1		;8 bit only
	clrf	A0
	rrf	A1		;Divide the result by 4
	rrf	A0		; Means 6 MSBs stay in A1 and 
	rrf	A1		;  LSBs are at top end of A0
	rrf	A0
	movf	A1, W		;
	andlw	0x3F		;Mask off any carry bits shifted in 
	movwf	CCPR1L		;At this PWM rate, only 8 bit setting is allowed, 6 in CCPR1L
				;  2 LSBs in CCP1CON[5,4]
	rrf	A0
	rrf	A0, W
	andlw	b'00110000'	;
	iorlw	b'00001100'	;PWM Mode
	movwf	CCP1CON		;Store 2 lowest bits
			;Increment the DDS
	movf	ToneLo, W	
	addwf	FreqLo

	movf	ToneMe, W
	btfsc	STATUS, C
	incfsz	ToneMe, W
	addwf	FreqMe	

	movf	ToneHi, W
	btfsc	STATUS, C
	incfsz	ToneHi, W
	addwf	FreqHi
				;Increment the timing counter
	incf	IntCount	
	movlw	INTSPERTIME
	subwf	IntCount, W	
	btfsc	STATUS, C	
	clrf	IntCount	

	btfsc	STATUS, Z	
	incf	TimCount			;Counts at Basic timing interval 5ms
	
	btfsc	ModeFlag
	goto	UpdateChirp			;141 clocks to here

	incf	RampCount		;Only used during tone mode, 16ms to give full rise at 7.8kHz
	btfsc	RampCount, 7
	decf	RampCount		;Count 0 - 127 and saturate there

	movlw	HIGH(RampTable)
	movwf	PCLATH

	rrf	RampCount, W		;Max count 127,  divide by 4
	movwf	iTemp
	rrf	iTemp, W
	andlw	0x1F

	btfss	KeyState
	sublw	d'31'			;Change direction of ramping

	call	RampTable
	movwf	Amplitude

	goto	OutInt
;.................
UpdateChirp			;Update the chirp frequency
	movlw	CHRATE & 0xFF			;Add chirp increment to Tone
	addwf	ToneLo

	movlw	(CHRATE >> d'8') & 0xFF
	btfsc	STATUS, C
	movlw	((CHRATE >> d'8') & 0xFF) + 1
	addwf	ToneMe

	movlw	(CHRATE >> d'16') & 0xFF
	btfsc	STATUS, C
	movlw	((CHRATE >> d'16') & 0xFF) + 1
	addwf	ToneHi
						;Test for maximum freq
	movf	ToneLo, W	
	sublw	(ENDFREQ & 0xFF)	

	movf	ToneMe, W
	btfss	STATUS, C
	incfsz	ToneMe, W
	sublw	(ENDFREQ >> d'8') & 0xFF	

	movf	ToneHi, W
	btfss	STATUS, C
	incfsz	ToneHi, W
	sublw	(ENDFREQ >> d'16') & 0xFF	

	btfsc	STATUS, C		;  [null] = ENDFREQ - Tone,  when -ve, C=0, redo chirp
	goto	OutInt

	movlw	(STARTFREQ >> d'16') & 0xFF
	movwf	ToneHi
	movlw	(STARTFREQ >> d'8') & 0xFF
	movwf	ToneMe
	movlw	STARTFREQ & 0xFF
	movwf	ToneLo

	movlw	d'250'
	movwf	Amplitude


OutInt
	movf	PCLcache, W	;restore PCLATH
	movwf	PCLATH
	swapf   StsReg , W      ;Restore STATUS
	movwf   STATUS
	swapf   Wreg            ;Restore W
	swapf   Wreg , W        ;

	bcf	STROBE
	retfie			;169 clocks on/off, 179 chirping.    64 * postscale permitted

;---------------------------------------------------------
Startup         
    	BANKSEL	ANSEL
	clrf	ANSEL			;WHY WHY WHY WHY WHY !!!!!!!!!!!!!!!!!!!!!
 
    	movlw   b'00001010'     	;0 STROBE, 1 MODE SW.   These will swap I/O in Prog mode
    	movwf   TRISIO           	;2 PWM, 3 KEY, 4 LED, 5 PTT
	movwf	WPU  
    	movlw   b'00000000'      	;Pull-ups
    	movwf   OPTION_REG

	movlw	0x3F			;PWM Period = Fclk/64/4 = 7812.5Hz
	movwf	PR2			;

	bsf	PIE1 , TMR2IE		;TMR2 overflow
    	BANKSEL	GPIO

	movlw	b'00001100'
	movwf	CCP1CON		;PWM Mode

	movlw	b'00011100'	;TMR2 On, prescale 1, postscale 4
	movwf	T2CON
	clrf	CCPR1L		;Duty cycle to zero

	clrf	Flags

	movlw	HIGH(SineTabContents)
	movwf	PCLATH		;Sine Table

	clrf	DelCount
	clrf	DelCount2

	movlw	(TONE2FREQ >> d'16') & 0xFF	;Startup at middle tone
	movwf	ToneHi
	movlw	(TONE2FREQ >> d'8') & 0xFF
	movwf	ToneMe
	movlw	TONE2FREQ & 0xFF
	movwf	ToneLo

	bsf	INTCON, PEIE		;External Interrupt enable
	bsf	INTCON, GIE		;Start it running
;.......................................
MainLoop		;Inputs KEY, MODESW.  Flags, ModeFlag, ManualKey, KeyState
			;First task, monitor key for manual operation 

	btfsc	MODESW		;If pressed, goes into chirp mode
	goto	NoModeChange
			;MODESW pressed = chirp and Tx on
	bsf	ModeFlag	
	bsf	LED
	bsf	PTT
NoModeChange
	btfsc	KEY		;Test for key down (= 0)
	goto	KeyUp

	bcf	ModeFlag	;Keypress cancels chirp

	movlw	(TONE2FREQ >> d'16') & 0xFF	;use Middle tone
	movwf	ToneHi
	movlw	(TONE2FREQ >> d'8') & 0xFF
	movwf	ToneMe
	movlw	TONE2FREQ & 0xFF
	movwf	ToneLo
	clrf	ToneCount


	btfss	MODESW		;Both pressed tri-tone mode
	goto	SeqTone

	btfsc	KeyState
	goto	MainLoop	;If already down, do nothing more

	bsf	KeyState	;First Press
	clrf	RampCount	
	bsf	LED
	bsf	PTT
	clrf	TimCount	;And resets TimCount for MOX

	goto	MainLoop
;..........
KeyUp
	btfsc	ModeFlag	;Cleared on any key down
	goto	MainLoop

	btfss	KeyState	;If already up at this stage, look for timeout
	goto	CheckMOX
			;If newly released
	bcf	KeyState
	clrf	RampCount
	clrf	TimCount
	goto	MainLoop
;.............	
CheckMOX
	movlw	MOX		;MOX timeout in manual CW mode only
	subwf	TimCount, W
	btfss	STATUS, C	
	goto	MainLoop
				;At TimCount threshold, go back to SBY.
	bcf	LED
	bcf	PTT
	bcf	KeyState

	goto	MainLoop	
;................
SeqTone				;Three sequential tones
	bcf	KeyState	;First, wait for key up and ModeSw to go high
	btfss	MODESW
	goto	SeqTone
	btfss	KEY
	goto	SeqTone

	bcf	ModeFlag	;Single tone
	bsf	PTT
	bsf	LED
	bsf	KeyState	;Sets amplitude to max
	clrf	ToneCount

SeqToneLoop
	movlw	CYCLETIM	;Wait for timer
	subwf	TimCount, W
	btfss	STATUS, C		
	goto	SeqToneLoop
	clrf	TimCount

	bcf	STATUS, C
	rlf	ToneCount, W
	addwf	ToneCount, W	;Multiply by 3 (bytes per freq word)
	andlw	0x07		
	movwf	Temp

	movlw	HIGH(LoadTones)
	movwf	PCLATH
	
	movf	Temp, W
	call	LoadTones	;PCLATH was set same as ramptable when exiting interrupt
	movwf	ToneHi

	movlw	1
	addwf	Temp, W
	call	LoadTones
	movwf	ToneMe

	movlw	2
	addwf	Temp, W
	call	LoadTones	;PCLATH was set same as ramptable when exiting interrupt
	movwf	ToneLo

	incf	ToneCount
	movlw	3
	subwf	ToneCount, W	;Counts 0 - 2
	btfsc	STATUS, C
	clrf	ToneCount


	btfss	KEY
	goto	OutPulse	;KeyDown resets


	goto	SeqToneLoop
		
;......
OutPulse	
	bcf	LED
	bcf	PTT
	bcf	KeyState

	goto	MainLoop
;==============================================
;---------------------------------------------
Multiply8       ;   Q0/1 = A * B
        clrf    Q1     
        clrf    Q0      
        movlw   8        
        movwf   Counter   
MultLoop
        movf    B0, W      
        bcf     STATUS, C   
        rrf     A0         
        btfsc   STATUS, C 
        addwf   Q1     
        rrf     Q1   
        rrf     Q0    
        decfsz  Counter    
        goto    MultLoop   

        return   
;-------------------------------------


;=================================================
 ORG  0x2C0
LoadTones	;This must be in the same page as RampTable
	andlw	0x0F
	addwf	PCL
	dt	(TONE1FREQ >> d'16') & 0xFF
	dt	(TONE1FREQ >> d'8') & 0xFF
	dt	TONE1FREQ & 0xFF

	dt	(TONE2FREQ >> d'16') & 0xFF
	dt	(TONE2FREQ >> d'8') & 0xFF
	dt	TONE2FREQ & 0xFF

	dt	(TONE3FREQ >> d'16') & 0xFF
	dt	(TONE3FREQ >> d'8') & 0xFF
	dt	TONE3FREQ & 0xFF

	dt	0,0,0,0,0,0, 0
;--------------------------------------
RampTable			;Amplitude Ramp		
	andlw	0x1F
	addwf   PCL	
  ;Raised Cosine
;	dt	0x00, 0x02, 0x05, 0x0A, 0x10, 0x16, 0x1E, 0x27, 0x31, 0x3C, 0x47, 0x53, 0x5F, 0x6C, 0x79, 0x85
;	dt	0x92, 0x9F, 0xAB, 0xB7, 0xC2, 0xCD, 0xD7, 0xE0, 0xE8, 0xEE, 0xF4, 0xF9, 0xFC, 0xFE, 0XFF, 0xFF
  ;Gaussian
	dt	d'0',  d'4',  d'6',  d'7',  d'10', d'12', d'15', d'19'
	dt	d'23', d'28', d'34', d'41', d'49', d'57', d'67', d'77'
	dt	d'88', d'101',d'113',d'127',d'141',d'155',d'169',d'183'
	dt	d'196',d'209',d'220',d'230',d'239',d'246',d'251',d'255'
;--------------------------

 ORG 0x2FF	;Adjust so TabContents falls EXACTLY on the next page boundary

SineTable
	addwf	PCL
SineTabContents
    dt    0x80,0x83,0x86,0x89,0x8C,0x8F,0x92,0x95
    dt    0x98,0x9C,0x9F,0xA2,0xA5,0xA8,0xAB,0xAE
    dt    0xB0,0xB3,0xB6,0xB9,0xBC,0xBF,0xC1,0xC4
    dt    0xC7,0xC9,0xCC,0xCE,0xD1,0xD3,0xD5,0xD8
    dt    0xDA,0xDC,0xDE,0xE0,0xE2,0xE4,0xE6,0xE8
    dt    0xEA,0xEC,0xED,0xEF,0xF0,0xF2,0xF3,0xF5
    dt    0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFC
    dt    0xFD,0xFE,0xFE,0xFF,0xFF,0xFF,0xFF,0xFF
    dt    0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFE,0xFE
    dt    0xFD,0xFC,0xFC,0xFB,0xFA,0xF9,0xF8,0xF7
    dt    0xF6,0xF5,0xF3,0xF2,0xF0,0xEF,0xED,0xEC
    dt    0xEA,0xE8,0xE6,0xE4,0xE2,0xE0,0xDE,0xDC
    dt    0xDA,0xD8,0xD5,0xD3,0xD1,0xCE,0xCC,0xC9
    dt    0xC7,0xC4,0xC1,0xBF,0xBC,0xB9,0xB6,0xB3
    dt    0xB0,0xAE,0xAB,0xA8,0xA5,0xA2,0x9F,0x9C
    dt    0x98,0x95,0x92,0x8F,0x8C,0x89,0x86,0x83
    dt    0x7F,0x7C,0x79,0x76,0x73,0x70,0x6D,0x6A
    dt    0x67,0x63,0x60,0x5D,0x5A,0x57,0x54,0x51
    dt    0x4F,0x4C,0x49,0x46,0x43,0x40,0x3E,0x3B
    dt    0x38,0x36,0x33,0x31,0x2E,0x2C,0x2A,0x27
    dt    0x25,0x23,0x21,0x1F,0x1D,0x1B,0x19,0x17
    dt    0x15,0x13,0x12,0x10,0x0F,0x0D,0x0C,0x0A
    dt    0x09,0x08,0x07,0x06,0x05,0x04,0x03,0x03
    dt    0x02,0x01,0x01,0x00,0x00,0x00,0x00,0x00
    dt    0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01
    dt    0x02,0x03,0x03,0x04,0x05,0x06,0x07,0x08
    dt    0x09,0x0A,0x0C,0x0D,0x0F,0x10,0x12,0x13
    dt    0x15,0x17,0x19,0x1B,0x1D,0x1F,0x21,0x23
    dt    0x25,0x27,0x2A,0x2C,0x2E,0x31,0x33,0x36
    dt    0x38,0x3B,0x3E,0x40,0x43,0x46,0x49,0x4C
    dt    0x4F,0x51,0x54,0x57,0x5A,0x5D,0x60,0x63
    dt    0x67,0x6A,0x6D,0x70,0x73,0x76,0x79,0x7C



;............................

;--------------------------------------------------
    cblock	0x40		;12F615  Only has 64 bytes of RAM, d'starting here
  	Wreg, StsReg, PCLcache
	Temp, Temp2, iTemp
	Counter
	IntCount, TimCount
	FreqHi, FreqMe, FreqLo
	ToneHi, ToneMe, ToneLo
	ToneCount
	Amplitude
	RampCount
	ChHi, ChMe, ChLo
	DelCount, DelCount2
 	Flags
	A0, A1, B0, B1
	Q0, Q1
    endc


	end