;KEYER5   G4JNt  Jan 2021
;  PWM manual CW, prestored message and Chirp   with COM Port setting of CW message.   Seq tones when no CW message is stored
;  Key input, Mode select pushbutton.

;PWM rate = Fxtal / 256 (8 bit PWM resolution).   Fosc = 8MHz INTRC
;Sampling rate = PWM rate /4 (Postscale = 4)  7.8kHz19:20 16/11/2020 
;24 Bit DDS, but tone defaults only 16 bit

;	Programming mode at boot up, same protocol as KEYER12F 
;       Auto selection of Serial Interface polarity
;	Q command added to set a selection of tone frequencies

; Enter programming mode when KEY and MODESW are both pressed at turn on
; LED Flashes Fast until 1s after both are released
;
	list		P=12F617
	include		"P12F617.INC"

	errorlevel 	2
	__config  0x3FC4  ;11 1111 0100 0010    WrtProt Off   BO EN  Not CP  MCLR Internal   PWRT EN  WDT Dis  HS Osc
						;0x3F42 for XT, 3FC4 for INTRC 8MHz
;--------------------- Define these user values -------------------------------------
	TONE1		=	d'494'
	TONE2		=	d'523'
	TONE3		=	d'587'
	STARTTONE	=	d'500'
	ENDTONE		=	d'2000'
	CHIRPRATE	=	d'3000'		;Hz/s
	TIMINT		=	8		;Timer interval, ms
	TMOX		=	d'800'		;CW MOX delay, ms
	PULSEONTIM	=	d'80'		;Pulse on period, ms   max value 128
	CYCLE		=	d'320'		;Rate of seq tone cycling ms
	
;Assembler values calculated using 32 bit {signed} integer arithmetic from user values, -- Do not touch
	FOSC		=	d'2000000'	
	FINT		=	FOSC / 4 / d'64'
	TONE1FREQ 	=	(TONE1 * d'1048576') / FINT * d'16'
	TONE2FREQ 	=	(TONE2 * d'1048576') / FINT * d'16'
	TONE3FREQ 	=	(TONE3 * d'1048576') / FINT * d'16'
	STARTFREQ 	=	(STARTTONE * d'1048576') / FINT * d'16'	;  32 bit Arithmetic
	ENDFREQ 	=	(ENDTONE * d'1048576') / FINT * d'16'
	CHRATE		=	((CHIRPRATE * d'65536') / FINT) * d'256' / FINT
	INTSPERTIME	=	(TIMINT * FINT) / d'1000'
	MOX		=	TMOX / TIMINT
	PULSEON		=	PULSEONTIM / TIMINT
	CYCLETIM	=	CYCLE / TIMINT

	#define		SERIN	GPIO, 0		;Shared with
	#define		STROBE	GPIO, 0		;Keep in for legacy
	#define		MODESW	GPIO, 1		;Shared with:
	#define		SEROUT	GPIO, 1
;			PWM	GPIO, 2
	#define		KEY	GPIO, 3		;Shared with PGM, so needs diodes
	#define		LED	GPIO, 4		;
	#define		PTT	GPIO, 5


	#define 	ModeFlag	Flags, 1	;0 = Tone, 1 = Chirp
	#define		ManualKey	Flags, 2
	#define		KeyState	Flags, 3
	#define		SaveTermFlag	Flags, 4	;Used in writing to flash
	#define		SerPol		Flags, 5	;1 = RS232 polarity, 0 = TTL

;=====================================
 org 0
boot    nop
        goto    Startup        		

;=========================
 org 4			;TMR2 Postscale interrupt rate.  Routine has to complete in 256 clocks total
	bsf	STROBE		;Changes 
	bcf	PIR1, TMR2IF

	movwf   Wreg            ;Save W
	swapf   STATUS , W      ;
	movwf   StsReg          ;Stave STATUS, without affecting it
	movf	PCLATH, W
	movwf	PCLcache	;Save PCLATH


	movlw	HIGH(SineTabContents)
	movwf	PCLATH

	movf	FreqHi, W
	call	SineTable

	movwf	A0
	movf	Amplitude, W
	movwf	B0
	call	Multiply8	;87 clocks, constant delay routine, MSB result in Q1

	bcf	STATUS, C
	rrf	Amplitude, W	;Add [128 - Amplitude/2] to normalise
	sublw	0x80		;   the level to mid point
	addwf	Q1, W

	movwf	A1		;8 bit only
	clrf	A0
	rrf	A1		;Divide the result by 4
	rrf	A0		; Means 6 MSBs stay in A1 and 
	rrf	A1		;  LSBs are at top end of A0
	rrf	A0
	movf	A1, W		;
	andlw	0x3F		;Mask off any carry bits shifted in 
	movwf	CCPR1L		;At this PWM rate, only 8 bit setting is allowed, 6 in CCPR1L
				;  2 LSBs in CCP1CON[5,4]
	rrf	A0
	rrf	A0, W
	andlw	b'00110000'	;
	iorlw	b'00001100'	;PWM Mode
	movwf	CCP1CON		;Store 2 lowest bits
			;Increment the DDS
	movf	ToneLo, W	
	addwf	FreqLo

	movf	ToneMe, W
	btfsc	STATUS, C
	incfsz	ToneMe, W
	addwf	FreqMe	

	movf	ToneHi, W
	btfsc	STATUS, C
	incfsz	ToneHi, W
	addwf	FreqHi
				;Increment the timing counter
	incf	IntCount	
	movlw	INTSPERTIME
	subwf	IntCount, W	
	btfsc	STATUS, C	
	clrf	IntCount	

	btfsc	STATUS, Z	
	incf	TimCountLo			;Counts at Basic timing interval 5ms
	
	btfsc	ModeFlag
	goto	UpdateChirp			;141 clocks to here


	incf	RampCount		;Only used during tone mode, 16ms to give full rise at 7.8kHz
	btfsc	RampCount, 7
	decf	RampCount		;Count 0 - 127 and saturate there

	movlw	HIGH(RampTable)
	movwf	PCLATH

	rrf	RampCount, W		;Max count 127,  divide by 4
	movwf	iTemp
	rrf	iTemp, W
	andlw	0x1F

	btfss	KeyState
	sublw	d'31'			;Change direction of ramping

	call	RampTable
	movwf	Amplitude

	goto	OutInt
;.................
UpdateChirp			;Update the chirp frequency
	movlw	CHRATE & 0xFF			;Add chirp increment to Tone
	addwf	ToneLo

	movlw	(CHRATE >> d'8') & 0xFF
	btfsc	STATUS, C
	movlw	((CHRATE >> d'8') & 0xFF) + 1
	addwf	ToneMe

	movlw	(CHRATE >> d'16') & 0xFF
	btfsc	STATUS, C
	movlw	((CHRATE >> d'16') & 0xFF) + 1
	addwf	ToneHi
						;Test for maximum freq
	movf	ToneLo, W	
	sublw	(ENDFREQ & 0xFF)	

	movf	ToneMe, W
	btfss	STATUS, C
	incfsz	ToneMe, W
	sublw	(ENDFREQ >> d'8') & 0xFF	

	movf	ToneHi, W
	btfss	STATUS, C
	incfsz	ToneHi, W
	sublw	(ENDFREQ >> d'16') & 0xFF	

	btfsc	STATUS, C		;  [null] = ENDFREQ - Tone,  when -ve, C=0, redo chirp
	goto	OutInt

	movlw	(STARTFREQ >> d'16') & 0xFF
	movwf	ToneHi
	movlw	(STARTFREQ >> d'8') & 0xFF
	movwf	ToneMe
	movlw	STARTFREQ & 0xFF
	movwf	ToneLo

	movlw	d'250'
	movwf	Amplitude


OutInt
	movf	PCLcache, W	;restore PCLATH
	movwf	PCLATH
	swapf   StsReg , W      ;Restore STATUS
	movwf   STATUS
	swapf   Wreg            ;Restore W
	swapf   Wreg , W        ;

	bcf	STROBE
	retfie			;169 clocks on/off, 179 chirping.    64 * postscale permitted

;---------------------------------------------------------
Startup
    	BANKSEL	ANSEL
	clrf	ANSEL			;WHY WHY WHY WHY WHY !!!!!!!!!!!!!!!!!!!!!
 
    	movlw   b'00001010'     	;0 STROBE, 1 MODE SW.   These will swap I/O in Prog mode
    	movwf   TRISIO           	;2 PWM, 3 KEY, 4 LED, 5 PTT
	movwf	WPU  
    	movlw   b'00000000'      	;Pull-ups
    	movwf   OPTION_REG

	movlw	0x3F			;PWM Period = Fclk/64/4 = 7812.5Hz
	movwf	PR2			;

	bsf	PIE1 , TMR2IE		;TMR2 overflow
    	BANKSEL	GPIO

	movlw	b'00001100'
	movwf	CCP1CON		;PWM Mode

	movlw	b'00011100'	;TMR2 On, prescale 1, postscale 4
	movwf	T2CON
	clrf	CCPR1L		;Duty cycle to zero

	clrf	Flags
	clrf	GPIO

	movlw	HIGH(SineTabContents)
	movwf	PCLATH		;Sine Table

	clrf	DelCount
	clrf	DelCount2
	clrf	RampCount

	movlw	HIGH(ToneStore)
	movwf	Temp2

	movlw	LOW(ToneStore)
	call	ReadFlash
	movwf	ToneFreqHi
	movwf	ToneHi

	movlw	LOW(ToneStore + 1)
	call	ReadFlash
	movwf	ToneFreqMe
	movwf	ToneMe

	movlw	LOW(ToneStore + 2)
	call	ReadFlash
	movwf	ToneFreqLo
	movwf	ToneLo

	movlw	PULSEON	
	movwf	CWSpeed

	btfsc	MODESW			;MODESW and KEY both pressed at turn on 
	goto	NormalStart		;  is ProgMode
	btfss	KEY
	goto	ProgMode
NormalStart
	bsf	INTCON, PEIE		;External Interrupt enable
	bsf	INTCON, GIE		;Start it running
;.......................................
MainLoop		;Inputs KEY, MODESW.  Flags, ModeFlag, ManualKey, KeyState
			;First task, monitor key for manual operation 

	btfsc	MODESW		;If pressed, goes into chirp mode
	goto	NoModeChange
				;MODESW pressed = chirp and Tx on
	bsf	ModeFlag	
	bsf	LED
	bsf	PTT
NoModeChange
	btfsc	KEY		;Test for key down (= 0)
	goto	KeyUp

	bcf	ModeFlag	;Keypress cancels chirp

	movf	ToneFreqHi, W		;Set CW Tone
	movwf	ToneHi
	movf	ToneFreqMe, W
	movwf	ToneMe
	movf	ToneFreqLo, W
	movwf	ToneLo

	btfss	MODESW		;Both pressed, stored CW or seq tones
	goto	SendCWMsg

	btfsc	KeyState
	goto	MainLoop	;If already down, do nothing more

	bsf	KeyState	;First Press
	clrf	RampCount	
	bsf	LED
	bsf	PTT
	clrf	TimCountLo	;And resets TimCountLo for MOX

	goto	MainLoop
;..........
KeyUp
	btfsc	ModeFlag	;Cleared on any key down
	goto	MainLoop

	btfss	KeyState	;If already up at this stage, look for timeout
	goto	CheckMOX
			;If newly released
	bcf	KeyState
	clrf	RampCount
	clrf	TimCountLo
	goto	MainLoop
;.............	
CheckMOX
	movlw	MOX		;MOX timeout in manual CW mode only
	subwf	TimCountLo, W
	btfss	STATUS, C	
	goto	MainLoop
				;At TimCountLo threshold, go back to SBY.
	bcf	LED
	bcf	PTT
	bcf	KeyState
	goto	MainLoop	
;........................
SeqTone				;Three sequential tones
	bcf	KeyState	;First, wait for key up and ModeSw to go high
	btfss	MODESW
	goto	SeqTone
	btfss	KEY
	goto	SeqTone

	bcf	ModeFlag	;Single tone
	bsf	PTT
	bsf	LED
	bsf	KeyState	;Sets amplitude to max
	clrf	ToneCount

SeqToneLoop
	movlw	CYCLETIM	;Wait for timer
	subwf	TimCountLo, W
	btfss	STATUS, C		
	goto	SeqToneLoop
	clrf	TimCountLo

	bcf	STATUS, C
	rlf	ToneCount, W
	addwf	ToneCount, W	;Multiply by 3 (bytes per freq word)
	andlw	0x07		
	movwf	Temp
 
	movlw	HIGH(LoadTones)
	movwf	PCLATH
	
	movf	Temp, W
	call	LoadTones	;PCLATH was set same as ramptable when exiting interrupt
	movwf	ToneHi

	movlw	1
	addwf	Temp, W
	call	LoadTones
	movwf	ToneMe

	movlw	2
	addwf	Temp, W
	call	LoadTones	;PCLATH was set same as ramptable when exiting interrupt
	movwf	ToneLo

	incf	ToneCount
	movlw	3
	subwf	ToneCount, W	;Counts 0 - 2
	btfsc	STATUS, C
	clrf	ToneCount


	btfss	KEY
	goto	OutPulse	;KeyDown resets
	goto	SeqToneLoop
		
;......
OutPulse	
	bcf	LED
	bcf	PTT
	bcf	KeyState

	goto	MainLoop
;......................
SendCWMsg
	clrf    CWMsgCnt
	movlw 	HIGH(CWContents)
	movwf	Temp2
	movlw	LOW(CWContents)
	call    ReadFlash	;Test for terminator at start = no CW message, send seq tone

	addlw   0
	btfsc   STATUS, Z
	goto	SeqTone

	movf	ToneFreqHi, W		;Set CW Tone
	movwf	ToneHi
	movf	ToneFreqMe, W
	movwf	ToneMe
	movf	ToneFreqLo, W
	movwf	ToneLo

	movlw   " "
	call    SendCW          ;Initial space
CWLoop
	movlw 	HIGH(CWContents)
	movwf	Temp2

	movf    CWMsgCnt, W
	addlw	LOW(CWContents)
	call    ReadFlash
	addlw   0
	btfsc   STATUS, Z	;Test for terminating zero
	goto	SendCWMsg	;And if so, restart

	call    SendCW
	incf    CWMsgCnt

	btfsc	KEY		;Key down terminates CW message loop
	goto    CWLoop
;...........
OutMsg
	bcf	LED
	bcf	PTT
	bcf	KeyState
	goto	MainLoop
;==============================================
SendCW          ;Sends a single CW character
    movwf   CWChar
    btfsc   CWChar, 7       ;Test for control code
    goto    DecodeToken

    movlw   HIGH(CWTable)
    movwf   PCLATH

    movf    CWChar, W
    addlw   0xE0     ;Subtract 32 (add 256 - 32) to point to table
    call    CWTable

    movwf   CWChar      ;Doesn't affect STATUS so...
    movf    CWChar      ;  check for invalid returned code here
    btfsc   STATUS , Z      
    goto    NoChar

    movlw   0x07    
    andwf   CWChar , W      ;Extract element count into W
    movwf   Counter     
    movlw   6           
    subwf   Counter , W     ;W = Elcount - 6.  If +ve/0 then C=1, > 5 els
    btfss   STATUS , C
    goto    Send5
    
    rrf     Counter     ;Puts LSB into C as 110 = .  111 = -
    call    SendEle     ;First  . or -  for 6 element characters
    movlw   5
    movwf   Counter     ;Force count to 5 for rest of the elements
Send5           
    btfss   KEY		;Quick getout if key is pressed
    return   	

    rlf     CWChar      ;Element to be sent into C.
    call    SendEle     ;   count already in Counter
    decfsz  Counter    
    goto    Send5
NoChar                  ;Inter word gap, and invalid codes.
    bcf     KeyState    ;Just in case
    bcf     LED
    call    CWDelay     
    call    CWDelay
    return
;.........
DecodeToken
    movlw   HIGH(CWTable)
    movwf   PCLATH
    btfss   CWChar, 6   ;B6 0 = Delay mode, 1 = WPM
    goto    ProgDelay

    movf    CWChar, W   ;11xxxsss  sss = WPM code
    andlw   0x07        ;Mask out speed control bits
    call    CWCtrlTable
    movwf   CWSpeed
    return
;.............
ProgDelay

    btfss   CWChar, 5   ;Set Tx/Rx line
    bcf     PTT
    btfsc   CWChar, 5
    bsf     PTT

    btfss   CWChar, 4   ;Set keying line
    bcf     KeyState
    btfsc   CWChar, 4
    bsf     KeyState
    clrf    RampCount

    bsf     LED         ;LED off if delay with Key up or in Rx mode
    btfss   KeyState
    bcf     LED
    btfss   CWChar, 5   ;
    bcf     LED

    movf    CWChar, W
    andlw   0x07
    addlw   0x08        	;Point to delay settings in table
    call    CWCtrlTable		;Return with number of seconds of delay in W

	movwf	DelCount
OuterDelLoop

	clrf	TimCountLo
InnerDelLoop			;Basic one second delay
	btfss	KEY
	goto	QuickOut

	movlw	d'125'
	subwf	TimCountLo, W
	btfss	STATUS, C
	goto	InnerDelLoop	

	decfsz	DelCount
	goto	OuterDelLoop
QuickOut
	bcf	KeyState
	clrf    RampCount
	bcf	LED
	bsf	PTT		
 	return
;-----------------------------------
SendEle
	bsf	KeyState
	clrf	RampCount	
	bsf	LED
	btfss	STATUS, C
	goto	SendDot
	call	CWDelay
	call	CWDelay
SendDot
	call	CWDelay

	bcf	KeyState
	clrf	RampCount	
	bcf	LED
	call	CWDelay

	return
;------------------------------------------
CWDelay
	clrf	TimCountLo
DeLoop
	movf	CWSpeed, W
	subwf	TimCountLo, W
	btfss	STATUS, C		
	goto	DeLoop	

	return
;--------------------------------------
;---------------------------------------------
Multiply8       ;   Q0/1 = A * B
        clrf    Q1     
        clrf    Q0      
        movlw   8        
        movwf   MultCount   
MultLoop
        movf    B0, W      
        bcf     STATUS, C   
        rrf     A0         
        btfsc   STATUS, C 
        addwf   Q1     
        rrf     Q1   
        rrf     Q0    
        decfsz  MultCount    
        goto    MultLoop   

        return   

;-------------------------------------
ReadFlash				;Read defined memory location as byte
	bcf	INTCON, GIE
	BANKSEL	PMADRL	
	movwf	PMADRL		;Low address byte in W,  high byte defined from label

 	BANKSEL	Temp2
 	movf	Temp2, W

 	BANKSEL	PMADRH
 	movwf	PMADRH
 
	bsf	PMCON1, RD	;Do the read
	nop			;This can be replaced by an instruction;  any ideas?
	nop			;This one CAN'T be.
	movf	PMDATL, W	;Low Byte contents returned in W.
	BANKSEL	GPIO		;As good as anything to get back to Page 0
	bsf	INTCON, GIE
	return
;---------------------------------------
SaveFlash			;Saves contents of the buffer up to zero terminator
	BANKSEL	PMADRL		;Only entered in ProgMode so interrupt won't be running
	movlw	HIGH(CWContents)
	movwf	PMADRH
	movlw	LOW(CWContents)
	movwf	PMADRL		;
	
	movlw	BUFFER
	movwf	FSR
SaveLoop
	movf	INDF, W		;
	movwf	PMDATL
	incf	FSR

	movf	PMDATL		;Test for terminator
	btfsc	STATUS, Z	;Freeze buffer at zero terminator point
	decf	FSR		; so INDF can be tested for zero to end

	clrf	PMDATH		;Only saving 8 bit data
	bsf	PMCON1, WREN

	movlw	0x55
	movwf	PMCON2
	movlw	0xAA
	movwf	PMCON2
	bsf	PMCON1, WR
	nop
	nop
	bcf	PMCON1, WREN

	movf	PMADRL, W
	incf	PMADRL
	andlw	0x03
	sublw	0x03
	btfss	STATUS, Z
	goto	SaveLoop	;Saves 4 bytes

	movf	PMDATL
	btfss	STATUS, Z
	goto	SaveLoop	;Gets out when zero terminator is found

	BANKSEL	GPIO		

	return
;---------------------------------------
SaveFreq		;Do separately as there's only 3 (4 incl dummy) bytes to save
	BANKSEL	PMADRL		;Only entered in ProgMode so interrupt won't be running
	movlw	HIGH(ToneStore)
	movwf	PMADRH
	movlw	LOW(ToneStore)
	movwf	PMADRL		;
	
	movlw	ToneFreqHi	;Saved as Hi, Med, Low
	movwf	FSR
SaveFreqLoop
	movf	INDF, W		;
	movwf	PMDATL
	incf	FSR

	clrf	PMDATH		;Only saving 8 bit data
	bsf	PMCON1, WREN

	movlw	0x55
	movwf	PMCON2
	movlw	0xAA
	movwf	PMCON2
	bsf	PMCON1, WR
	nop
	nop
	bcf	PMCON1, WREN

	movf	PMADRL, W
	incf	PMADRL
	andlw	0x03
	sublw	0x03
	btfss	STATUS, Z
	goto	SaveFreqLoop	;Saves 4 bytes, so one dummy

	BANKSEL	GPIO		

	return


;---------------------------------------
;================~~~~~~~~~~~~~~~~======================~~~~~~~~~~~~~~~~~~~~~~===================~~~~~~~~~
ProgMode		;Here when KEY and MODESW both pressed at turn on
	movlw	d'15'		;Fast flash LED for 1.5s and until both lines go high
	movwf	Counter		; go high to show progmode
ProgSLoop	
	bcf	LED
	movlw	d'50'
	call	DelayNms
	bsf	LED
	movlw	d'50'
	call	DelayNms
	decfsz	Counter
	goto	ProgSLoop

	btfss	KEY		;Wait for key to go high
	goto	ProgMode
				;GPIO0/1 switched I/O as MODESW needs to become an output
	BANKSEL TRISIO		;  so it isn't loaded by COM port
	movlw   b'00001001'     ;0 SERIN, 1 SEROUT
    	movwf   TRISIO          ;
	BANKSEL	GPIO

	movlw	d'100'		
	call	DelayNms

	bcf	SerPol		;Test resting polarity of SERIN line  Low = RS232   High = TTL
	btfss	SERIN		;O/C pulled high so default is TTL
	bsf	SerPol		;If low it's RS232	

	btfss	SerPol		;Set SEROUT to the correct polarity
	bsf	SEROUT		
	btfsc	SerPol
	bcf	SEROUT		;Then delay a bit to clear any BREAK state in terminal
	movlw	d'250'		
	call	DelayNms

	call	CrLf
	call	CrLf
	movlw   LOW(IntroMsg)
	call	DispMsg
ProgDoLoop
	movlw   LOW(MenuPrompt)
	call    DispMsg
	call	Qprompt

	call    WaitData        ;Get D / E / S commands
	movf	RxChar, W
	call	Send232
	call	CrLf
 
	movlw   "D"
	subwf   RxChar, W
	btfsc   STATUS, Z
	goto    Display

	movlw   "Q"
	subwf   RxChar, W
	btfsc   STATUS, Z
	goto    NewTone

	movlw   "E"
	subwf   RxChar, W
	btfsc   STATUS, Z
	goto    EnterMsg
	
	movlw   "S"
	subwf   RxChar, W
	btfsc   STATUS, Z
	goto    ReStart

	goto    ProgDoLoop      ;Any other character
;.................
ReStart
	call	CrLf
	call	CrLf

	BANKSEL	TRISIO			;Restore the I/O pins
   	movlw   b'00001010'     	;0 STROBE, 1 MODE SW.   These will swap I/O in Prog mode
    	movwf   TRISIO           	;2 PWM, 3 KEY, 4 LED, 5 PTT
	movwf	WPU  
	BANKSEL	GPIO

	bsf	INTCON, PEIE		;External Interrupt enable
	bsf	INTCON, GIE		;Start it running

	goto	SendCWMsg
;-----------------
Display
    	call    CrLf
    	movlw   " "
    	call    Send232
    	movlw   "C"
    	call    Send232
    	movlw   "W"
    	call    Send232
    	movlw   " "
    	call    Send232
    	movlw   " "
    	call    Send232
    	movlw   0x22
    	call    Send232

    	movlw   0
    	movwf   CWMsgCnt
DispCWLoop
	movlw 	HIGH(CWContents)
	movwf	Temp2
    	movf    CWMsgCnt, W
	addlw	LOW(CWContents)
   	call    ReadFlash
    	addlw   0
    	btfsc   STATUS, Z       ;Test for terminator
    	goto    DoneDisp

    	movwf   Token           ;Save in case it is a Token
    	btfss   Token, 7
    	goto    NotToken
    	call    DispToken      

NotToken
    	call    Send232
    	incf    CWMsgCnt
    	goto    DispCWLoop
;...........
DoneDisp
    	movlw   0x22
    	call    Send232         ;Closing "
    	call    CrLf
    	call    CrLf
    	goto    ProgDoLoop
;-----------------------
EnterMsg                    ;Real time data entry and write to Flash via buffer
	movlw	BUFFER
	movwf	FSR		;Start of Buffer memory
	
	clrf    CWMsgCnt
	movlw   LOW(CWPrompt)
	call    DispMsg

	movlw   LOW(SpdMsg1)
	call    DispMsg
	movlw   LOW(SpdMsg2)
	call    DispMsg

	call	Qprompt

EnterCWMsgLoop
    call    WaitData
    movlw   0x0D
    subwf   RxChar, W
    btfsc   STATUS, Z
    goto    CWMsgStored

    movlw   0x08            ;Abort
    subwf   RxChar, W
    btfsc   STATUS, Z
    goto    Abort

    movlw   " "             ;Test for all other control characters
    subwf   RxChar, W       ;  and ignore them
    btfss   STATUS, C       ;If +ve or 0, C=1 > 0x31 so OK
    goto    EnterCWMsgLoop

    movlw   "<"
    subwf   RxChar, W
    btfsc   STATUS, Z
    goto    GetTokenInfo
    
    movf    RxChar, W
    call    Send232
    movf    RxChar, W
BackHere                    ;return from token entry decoding to this point
	movwf	INDF		;Save in buffer
	incf	FSR

	btfsc	FSR, 7		;Look for overrun to 0x80
	goto	OverRun

	incf	CWMsgCnt
	goto	EnterCWMsgLoop
;...........
OverRun
	decf	FSR
	clrf	INDF		;Add a terminator for safety
	movlw	LOW(OVRunPrompt)
	call	DispMsg
	goto    CWMsgStored
;................
Abort                    ;
	clrf	INDF		;Add terminators for safety
	incf	FSR
	clrf	INDF
	call	CrLf
	goto    Display
;..................
CWMsgStored
	movlw   0               ;Counter has already been incremented
	movwf	INDF		; so save terminator

;	call	CrLf		;Display data from RAM (tokens not decoded)
	movlw	BUFFER
	movwf	FSR
DispMemLoop
	movf	INDF, W
	iorlw	0
	btfsc	STATUS, Z
	goto	DoneShowEntry

	incf	FSR

	goto	DispMemLoop
;.......
DoneShowEntry
	call	CrLf
	call    SaveFlash        
	goto    Display     ;Return to main prog menu
;........................................
;CW Delay bit coding    10RKxddd   R - 0=Rx, K - 0=KeyUp  ddd = delay code
;CW WPM Bit Coding      11xxxsss  sss = WPM code      

GetTokenInfo   ; From RS232 eg <DTDA>   Delay, Transmit, key Down, duration A
    movlw   "<"     ;       or <WC>     WPM, speed C
    call    Send232         ;Echo back token intro
    movlw   0x80
    movwf   Token           ;Base value

    call    WaitData        ;First look for WPM or Delay code
    movlw   "W"
    subwf   RxChar, W
    btfsc   STATUS, Z
    goto    SetWPM

    movlw   "D"             ;Echo back what S/W has interpreted it as...
    call    Send232

    call    WaitData        ;Get Tx/Rx status
    movlw   "T"
    subwf   RxChar, W
    btfsc   STATUS, Z
    bsf     Token, 5        ;B5 = 0 Rx, 1 Tx

    movlw   "R"
    btfsc   Token, 5
    movlw   "T"
    call    Send232         ;Echo back what is actually stored

    call    WaitData        ;Get Key Up/Down
    movlw   "D"
    subwf   RxChar, W
    btfsc   STATUS, Z
    bsf     Token, 4        ;B4 = 0 Up, 1 Dn

    movlw   "U"
    btfsc   Token, 4
    movlw   "D"
    call    Send232         ;Echo back what is actually stored

    call    WaitData        ;Get delay code A-H
    addlw   0xBF            ;Subtract 0x41 to convert to 0 - 7
    andlw   0x07            ;Just in case
    addwf   Token

    addlw   "A"             ;Return code for what it actually is
    call    Send232
    goto    WaitCloseToken
;..........
SetWPM
    movlw   "W"
    call    Send232
    bsf     Token, 6

    call    WaitData        ;Get delay code A-H
    addlw   0xBF            ;Subtract 0x41 to convert to 0 - 7
    andlw   0x07            ;Just in case
    addwf   Token

    movf    Token, W
    andlw   0x07
    addlw   "A"
    call    Send232

WaitCloseToken
    call    WaitData        ;Wait for closing >
    movlw   ">"
    subwf   RxChar, W
    btfss   STATUS, Z
    goto    WaitCloseToken
    movlw   ">"
    call    Send232
    movf    Token, W
    goto    BackHere        ;Return with Token code in W
;-------------------------------------
NewTone
	movlw	LOW(ToneMsg)
	call	DispMsg
	call	Qprompt

	call	WaitData	;Get a letter A - H
	addlw	0xFF		;Convert to a value 0 - 7
	andlw	0x07
	movwf	Temp
	addwf	Temp	
	addwf	Temp		;Multiply by 3 as 3 bytes per freq	

	movlw	HIGH(ToneTable)
	movwf	PCLATH

	movf	Temp, W
	call	ToneTable	
	movwf	ToneFreqHi

	movf	Temp, W
	addlw	1
	call	ToneTable	
	movwf	ToneFreqMe

	movf	Temp, W
	addlw	2
	call	ToneTable	
	movwf	ToneFreqLo

	call	SaveFreq

	movf	RxChar, W	;recover the code entered
	call	Send232
	call	CrLf
	call	CrLf

	goto    ProgDoLoop

;===========  SUBROUTINES SPECIFIC TO PROG MODE BELOW HERE ============================
DelayNms
	movwf	DelCount2
DelayNLoop
	call	Delay1ms
	decfsz	DelCount2
	goto	DelayNLoop
	return
;----------------------------------------------
Delay1ms
	movlw	(FOSC / d'10000')	;Loop is 10.N clocks long
	movwf	DelCount
Del1msLoop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	decfsz	DelCount
	goto	Del1msLoop

	return
;---------------------------------------
CrLf                    ;Uses less space than storing in string
    movlw   0x0D
    call    Send232
    movlw   0x0A
    call    Send232
    return
;--------------------------------
Qprompt
	movlw	"?"
	call	Send232
	movlw	" "
	call	Send232
	return
;---------------------------

DispToken          ;If valid, decodes token byte to RS232 link
    movlw   "<"
    call    Send232
    btfss   Token, 6
    goto    ShowDelay

    movlw   "W"
    call    Send232
    movf    Token, W
    andlw   0x07
    addlw   "A"
    call    Send232

    goto    OutDispToken
;..........
ShowDelay
;CW Delay bit coding    10RKxddd   R - 0=Rx, K - 0=KeyUp  ddd = delay code
    movlw   "D"
    call    Send232

    movlw   "R"
    btfsc   Token, 5
    movlw   "T"
    call    Send232

    movlw   "U"
    btfsc   Token, 4
    movlw   "D"
    call    Send232

    movf    Token, W
    andlw   0x07
    addlw   "A"
    call    Send232

OutDispToken
    movlw   ">"             ;A Send232 command always follows the return
    return
;---------------------------
WaitData              		;Sit in here waiting for RS232 start bit
	btfss	SerPol
	goto	TestForTTL
	
	btfss	SERIN		;Test for RS232 start transition  0 >> 1
	goto	WaitData
	goto	TransFound	
;.............
TestForTTL
	btfsc	SERIN		;Test for TTL start transition   1 >> 0
	goto	WaitData
TransFound

	bcf     LED             ;Blink LED for each character entered
	call    HalfBitDelay    

	btfss	SerPol
	goto	TestForTTL2
	
	btfss	SERIN		
	goto	WaitData
	goto	TransFound2	
;.............
TestForTTL2
	btfsc	SERIN		
	goto	WaitData
TransFound2
    movlw   8               ;
    movwf   Counter
    clrf    RxChar          ; to here from transition = half bit
ByteLoop
    call    HalfBitDelay
    call    HalfBitDelay    
    bcf     STATUS , C     
    btfss   SERIN         
    bsf     STATUS , C     
    rrf     RxChar        
    decfsz  Counter
    goto    ByteLoop        ;this loop 2 * HalfBitDelay + 8

	btfss	SerPol		;Invert if TTL data
	comf	RxChar

    movf    RxChar , w      ;Returns with stop bit+  slack period
    movlw   "a"             ;Convert to Upper case
    subwf   RxChar, W       ;W = Char - "a"
    btfss   STATUS, C       ;if-ve (C=0) char < "a" so ignore
    goto    UpCaseDone

    movf    RxChar, W
    sublw   "z"             ;W = "z" - Char
    btfss   STATUS, C       ;if -ve (C=0)  char > "z" so ignore
    goto    UpCaseDone

    bcf     RxChar, 5       ;Reset bit 5 to force upper case
UpCaseDone
    movf    RxChar, W
    bsf     LED
    return                  ;received byte in RxChar and W
;-------------------------
HalfBitDelay                ;For RS232 1200 Baud link
    movlw   FOSC / d'12000' - 4         ;
    movwf   DelCount
HalfBaudDelLoop
    nop
    nop
    decfsz  DelCount
    goto    HalfBaudDelLoop
    return
;---------------------------
Send232             ;Enter with data in W, send it to PC
	movwf   Temp
	btfss	SerPol
	comf	Temp		;TTL polarity is inverted

	btfsc	SerPol		;1 = RS232, 0 = TTL
	bsf     SEROUT 		;start bit
	btfss	SerPol
	bcf	SEROUT

    call    HalfBitDelay
    call    HalfBitDelay        ;
    movlw   8
    movwf   Counter
    nop
    nop
SendDatLoop
    rrf     Temp        	;bit into C
    btfsc   STATUS , C
    bcf     SEROUT
    btfss   STATUS , C
    bsf     SEROUT         	;could have 2us jitter depending on 1/0
    call    HalfBitDelay
    call    HalfBitDelay    
    decfsz  Counter
    goto    SendDatLoop     	;loop 8 + HalfBitDelay long

	btfsc	SerPol		;1 = RS232, 0 = TTL
	bcf     SEROUT 		;Stop bit
	btfss	SerPol
	bsf	SEROUT
    call    HalfBitDelay    
    call    HalfBitDelay    
    call    HalfBitDelay    
    call    HalfBitDelay    ;Two stop bits
    return
;-------------------------

;Generating a message from data stored in programme memory
;               Uses AddrHold, MsgCount

DispMsg   ;Enter with address of Text in W  eg.  'MOVLW LOW(TextLabel)'
    movwf   AddrHold    ;Store offset of text string

    movlw   HIGH(MsgTable)
    movwf   PCLATH

    movlw   low(MsgTable)
    addlw   1
    subwf   AddrHold , W    ;W = AddrTextString - TableStart - 1
                            ;  = Offset of string start in table
MsgLoop
    movwf   MsgCount
    call    MsgTable
    andlw   0xFF            ;end of message
    btfsc   STATUS , Z
    goto    MsgDone

    call    Send232
    movf    MsgCount, W
    addlw   1
    goto    MsgLoop

MsgDone
    call    CrLf        ;Always put a CR LF at end, saves storing it in msg.
    return
;.........
;============================================
 ORG 0x400
MsgTable
    addwf   	PCL
IntroMsg    	dt "G4JNT Beacon Keyer" , 0
CWPrompt    	dt "Token Codes" , 0
MenuPrompt  	dt "[D]isplay [E]nter [S]end Fre[Q]",0
OVRunPrompt 	dt " MEM OVRN",0
SpdMsg1		dt "Delay seconds A-1 B-5 C-10 D-15 E-20 F-30 G-60 H-90",0
SpdMsg2		dt "WPM           A-6 B-8 C-10 D-12 E-15 F-20 G-24 H-30",0
ToneMsg		dt "Tone Code  A-432 B-528 C-645  D-789  E-964 F-1178 G-1440 H-1760",0

MsgTableEnd

    if ((DispMsg & 0xFF) >= (MsgTableEnd & 0xFF))
        ERROR    "Table code crosses page boundary"
        ERROR    "ADJUST START OF TABLE"
    endif


;=================================================

 org 0x4FF	;Adjust so TabContents falls EXACTLY on the next page boundary

SineTable
	addwf	PCL
SineTabContents
    dt    0x80,0x83,0x86,0x89,0x8C,0x8F,0x92,0x95
    dt    0x98,0x9C,0x9F,0xA2,0xA5,0xA8,0xAB,0xAE
    dt    0xB0,0xB3,0xB6,0xB9,0xBC,0xBF,0xC1,0xC4
    dt    0xC7,0xC9,0xCC,0xCE,0xD1,0xD3,0xD5,0xD8
    dt    0xDA,0xDC,0xDE,0xE0,0xE2,0xE4,0xE6,0xE8
    dt    0xEA,0xEC,0xED,0xEF,0xF0,0xF2,0xF3,0xF5
    dt    0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFC
    dt    0xFD,0xFE,0xFE,0xFF,0xFF,0xFF,0xFF,0xFF
    dt    0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFE,0xFE
    dt    0xFD,0xFC,0xFC,0xFB,0xFA,0xF9,0xF8,0xF7
    dt    0xF6,0xF5,0xF3,0xF2,0xF0,0xEF,0xED,0xEC
    dt    0xEA,0xE8,0xE6,0xE4,0xE2,0xE0,0xDE,0xDC
    dt    0xDA,0xD8,0xD5,0xD3,0xD1,0xCE,0xCC,0xC9
    dt    0xC7,0xC4,0xC1,0xBF,0xBC,0xB9,0xB6,0xB3
    dt    0xB0,0xAE,0xAB,0xA8,0xA5,0xA2,0x9F,0x9C
    dt    0x98,0x95,0x92,0x8F,0x8C,0x89,0x86,0x83
    dt    0x7F,0x7C,0x79,0x76,0x73,0x70,0x6D,0x6A
    dt    0x67,0x63,0x60,0x5D,0x5A,0x57,0x54,0x51
    dt    0x4F,0x4C,0x49,0x46,0x43,0x40,0x3E,0x3B
    dt    0x38,0x36,0x33,0x31,0x2E,0x2C,0x2A,0x27
    dt    0x25,0x23,0x21,0x1F,0x1D,0x1B,0x19,0x17
    dt    0x15,0x13,0x12,0x10,0x0F,0x0D,0x0C,0x0A
    dt    0x09,0x08,0x07,0x06,0x05,0x04,0x03,0x03
    dt    0x02,0x01,0x01,0x00,0x00,0x00,0x00,0x00
    dt    0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01
    dt    0x02,0x03,0x03,0x04,0x05,0x06,0x07,0x08
    dt    0x09,0x0A,0x0C,0x0D,0x0F,0x10,0x12,0x13
    dt    0x15,0x17,0x19,0x1B,0x1D,0x1F,0x21,0x23
    dt    0x25,0x27,0x2A,0x2C,0x2E,0x31,0x33,0x36
    dt    0x38,0x3B,0x3E,0x40,0x43,0x46,0x49,0x4C
    dt    0x4F,0x51,0x54,0x57,0x5A,0x5D,0x60,0x63
    dt    0x67,0x6A,0x6D,0x70,0x73,0x76,0x79,0x7C



;............................
;----------------------------
 org 0x600
CWTable           ;CW Characters in compressed form. 
    addwf   PCL
    
    dt   b'00000000'  ; [sp] 00
    dt   b'01011111'  ;  !   5F
    dt   b'10010110'  ;  "   96
    dt   b'00010101'  ;  #   15
    dt   b'00000000'  ;  $   00
    dt   b'00000000'  ;  %   00
    dt   b'01000101'  ;  &   45
    dt   b'11110110'  ;  '   F6
    dt   b'10110101'  ;  (   B5
    dt   b'01101111'  ;  )   6F
    dt   b'01000101'  ;  *   45
    dt   b'01010101'  ;  +   55
    dt   b'10011111'  ;  ,   9F     Coding for CW characters of up to
    dt   b'00001111'  ;  -   0F      6 elements.  Stored as :
    dt   b'10101110'  ;  .   AE     Bits 7 to 3 make first five CW elements
    dt   b'10010101'  ;  /   95     Reading  Left to Right 
    dt   b'11111101'  ;  0   FD     Coded as 0 = dit   1 = dah
    dt   b'01111101'  ;  1   7D     Bts 2-0 are number of elements except for 
    dt   b'00111101'  ;  2   3D     110 > 1st element is a dit  
    dt   b'00011101'  ;  3   1D     111 > 1st element is a dah       
    dt   b'00001101'  ;  4   0D
    dt   b'00000101'  ;  5   05
    dt   b'10000101'  ;  6   85
    dt   b'11000101'  ;  7   C5
    dt   b'11100101'  ;  8   E5
    dt   b'11110101'  ;  9   F5
    dt   b'11000111'  ;  :   C7
    dt   b'01010111'  ;  ;   47
    dt   b'00000000'  ;  <   00
    dt   b'10001101'  ;  =   8D
    dt   b'00000000'  ;  >   00
    dt   b'01100110'  ;  ?   66
    dt   b'11010110'  ;  @   D6
    dt   b'01000010'  ;  A   42
    dt   b'10000100'  ;  B   84
    dt   b'10100100'  ;  C   A4
    dt   b'10000011'  ;  D   83
    dt   b'00000001'  ;  E   01
    dt   b'00100100'  ;  F   24
    dt   b'11000011'  ;  G   C3
    dt   b'00000100'  ;  H   04
    dt   b'00000010'  ;  I   02
    dt   b'01110100'  ;  J   74
    dt   b'10100011'  ;  K   A3
    dt   b'01000100'  ;  L   44
    dt   b'11000010'  ;  M   C2
    dt   b'10000010'  ;  N   82
    dt   b'11100011'  ;  O   E3
    dt   b'01100100'  ;  P   64
    dt   b'11010100'  ;  Q   D4
    dt   b'01000011'  ;  R   43
    dt   b'00000011'  ;  S   03
    dt   b'10000001'  ;  T   81
    dt   b'00100011'  ;  U   23
    dt   b'00010100'  ;  V   14
    dt   b'01100011'  ;  W   63
    dt   b'10010100'  ;  X   94
    dt   b'10110100'  ;  Y   B4
    dt   b'11000100'  ;  Z   C4

CWCtrlTable
    	addwf   PCL
    	dt  d'25',d'19',d'15',d'12',d'10',d'8',d'6',d'5'		;Units of 8ms
;	     6       8     10     12    15    20    24   30  WPM
    	dt  d'1',d'5',d'10',d'15',d'20',d'30',d'60',d'90'		;Seconds

ToneTable 				
	addwf	PCL
        dt        0x0E, 0x27, 0xE1         ;  432Hz
        dt        0x11, 0x4D, 0x2F         ;  528Hz
        dt        0x15, 0x22, 0xA7         ;  645Hz
        dt        0x19, 0xDA, 0x9D         ;  789Hz
        dt        0x1F, 0x96, 0x9E         ;  964Hz
        dt        0x26, 0x99, 0xC8         ;  1178Hz
        dt        0x2F, 0x2F, 0x98         ;  1440Hz
        dt        0x39, 0xAB, 0xF3         ;  1760Hz

;=============================================
 ORG  0x690
LoadTones	;This must be in the same page as RampTable
	andlw	0x0F
	addwf	PCL
	dt	(TONE1FREQ >> d'16') & 0xFF
	dt	(TONE1FREQ >> d'8') & 0xFF
	dt	TONE1FREQ & 0xFF

	dt	(TONE2FREQ >> d'16') & 0xFF
	dt	(TONE2FREQ >> d'8') & 0xFF
	dt	TONE2FREQ & 0xFF

	dt	(TONE3FREQ >> d'16') & 0xFF
	dt	(TONE3FREQ >> d'8') & 0xFF
	dt	TONE3FREQ & 0xFF

	dt	0,0,0,0,0,0, 0
;.................
RampTable			;Amplitude Ramp		
	andlw	0x1F
	addwf   PCL	
  ;Raised Cosine
	dt	0x00, 0x02, 0x05, 0x0A, 0x10, 0x16, 0x1E, 0x27, 0x31, 0x3C, 0x47, 0x53, 0x5F, 0x6C, 0x79, 0x85
	dt	0x92, 0x9F, 0xAB, 0xB7, 0xC2, 0xCD, 0xD7, 0xE0, 0xE8, 0xEE, 0xF4, 0xF9, 0xFC, 0xFE, 0XFF, 0xFF
  ;Gaussian
;	dt	d'0',  d'4',  d'6',  d'7',  d'10', d'12', d'15', d'19'
;	dt	d'23', d'28', d'34', d'41', d'49', d'57', d'67', d'77'
;	dt	d'88', d'101',d'113',d'127',d'141',d'155',d'169',d'183'
;	dt	d'196',d'209',d'220',d'230',d'239',d'246',d'251',d'255'
;=====================================
 org 0x700		
ToneStore		;CW Tone use 4th dummy byte
	dt 	0x11, 0x4D, 0x2F, 0	; 1183271 == 528Hz

 org 0x710		;Write process does 16 at a time, so addresses need to be apart
CWContents		;CW Message up to ~50 chars stored here.
;	dt	b'11000101',"G4JNT UW KEYER",b'10110001', 0   	;<WE>    <DTDB>
	dt	0,0,0,0,0

	dt	"&&&&&&&& KEYER4 &&&&&&&&"		
;--------------------------------------------------
    cblock	0x20		;12F617 has 128 bytes RAM
  	Wreg, StsReg, PCLcache
	Temp, Temp2, iTemp
	Counter
	IntCount, TimCountLo
	FreqHi, FreqMe, FreqLo
	ToneHi, ToneMe, ToneLo
	ToneFreqHi, ToneFreqMe, ToneFreqLo
	ToneCount
	Amplitude
	RampCount
	DelCount, DelCount2
 	Flags
	A0, A1, B0, B1
	Q0, Q1, MultCount
	CWChar, CWSpeed, CWMsgCnt
	RxChar, Token, AutoEEADR
	MsgCount, AddrHold
	BUFFER			;Used as a label only. Must be last item as buffer grows down from here
    endc


	end